import React__default,{useState,useEffect}from'react';import {Spin,Row,Col,Button,Card,message}from'antd';import {useGetDurationTimeRecordsQuery,useGetDurationTimesheetsQuery,useCreateTimeRecordMutation,useUpdateTimeRecordMutation,useRemoveDurationTimeRecordsMutation,useCreateTimesheetMutation}from'@admin-layout/timetracker-core/lib/generated/react-hooks';import {useSplitTimeRecordsPerDay}from'../../../hooks/index.js';import {TabularCalendar}from'./TabularCalendar.js';import moment from'moment';import*as _ from'lodash';const filterTimeRecords = (records, filterOptions) => {
    const { selectedUser, selectedProject, approval, members } = filterOptions;
    if (!records)
        return [];
    const memStrArr = members.map((mem) => mem.userId);
    return records
        .filter((ev) => (ev.userId === selectedUser || selectedUser === '__all') &&
        (ev.projectId === selectedProject || selectedProject === '__all') &&
        (approval === undefined ? true : approval ? !!ev.timesheetId : !ev.timesheetId))
        .filter((record) => memStrArr.includes(record.userId));
};
const TabularCalendarWrapper = ({ projects, weekStart, selectedUser, members, selectedProject, setPathWeekStart, }) => {
    const [timeRecords, setTimeRecords] = useState([]);
    const { data: { getDurationTimeRecords } = {}, loading: loadingRecords, refetch, error, } = useGetDurationTimeRecordsQuery({
        variables: {
            startTime: weekStart,
            endTime: moment(weekStart).add(1, 'week'),
        },
    });
    const records = useSplitTimeRecordsPerDay(getDurationTimeRecords);
    const { data: approvalData, loading: loadingApproval, refetch: refetchApproval, } = useGetDurationTimesheetsQuery({
        variables: {
            start: moment(weekStart),
            end: moment(weekStart).add(1, 'week'),
        },
    });
    useEffect(() => {
        setTimeRecords(records.map((record) => (Object.assign(Object.assign({}, record), { isSplit: records.filter((data) => data.id === record.id).length > 1 }))));
    }, [records]);
    const reloadData = () => {
        refetch();
        refetchApproval();
    };
    useEffect(() => {
        reloadData();
    }, [weekStart]);
    const [createMutation, { loading: loadingCreate }] = useCreateTimeRecordMutation();
    const [updateMutation, { loading: loadingUpdate }] = useUpdateTimeRecordMutation();
    const [removeMutation, { loading: loadingRemove }] = useRemoveDurationTimeRecordsMutation();
    const [createTimesheetMutation, { loading: loadingCreateTimesheet }] = useCreateTimesheetMutation();
    const handleRemoveDuration = (pId) => {
        removeMutation({
            variables: {
                startTime: moment(weekStart),
                endTime: moment(weekStart).add(1, 'week'),
                projectId: pId,
            },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('Removed');
                }
                else {
                    console.log(errors[0].message);
                }
            }
        });
    };
    // create time record
    const createTimeRecord = (request) => {
        createMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('TimeRecord created');
                }
                else {
                    message.error(error.message);
                }
            },
        });
    };
    // update time record
    const updateTimeRecord = (recordId, request) => {
        updateMutation({
            variables: { recordId, request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimeRecords() { }
                        }
                    });
                    message.success('TimeRecord Updated');
                }
                else {
                    message.error(error.message);
                }
            }
        });
    };
    // create timeSheet
    const createTimesheet = (request) => {
        createTimesheetMutation({
            variables: { request },
            update: (cache, { data, errors }) => {
                if (data) {
                    cache.modify({
                        fields: {
                            getDurationTimesheets() { }
                        }
                    });
                    message.success('Timesheet Created');
                }
                else {
                    message.error(error.message);
                }
            },
        });
    };
    const memberTimesheets = (userId) => {
        return _.get(approvalData, 'getDurationTimesheets', []).filter((sheet) => sheet.userId === userId);
    };
    const projectsApproval = () => {
        let approvedSet = new Set();
        let unApprovedSet = new Set();
        filterTimeRecords(timeRecords, {
            selectedUser,
            selectedProject,
            members,
        }).forEach((record) => {
            if (!!record.timesheetId)
                approvedSet.add(record.projectId || '');
            else
                unApprovedSet.add(record.projectId || '');
        });
        return {
            approvals: Array.from(approvedSet.values()),
            unApprovals: Array.from(unApprovedSet.values()),
        };
    };
    const getProjectsMap = () => {
        const projectsMap = new Map();
        projects.forEach((project) => {
            projectsMap.set(project.id, project);
        });
        return projectsMap;
    };
    const onClickBack = (event) => {
        event.preventDefault();
        const newWeekStart = moment(weekStart).add('-1', 'week');
        setPathWeekStart(newWeekStart);
    };
    const onClickNext = (event) => {
        event.preventDefault();
        const newWeekStart = moment(weekStart).add('1', 'week');
        setPathWeekStart(newWeekStart);
    };
    const onClickToday = (event) => {
        event.preventDefault();
        const newWeekStart = moment().startOf('week');
        setPathWeekStart(newWeekStart);
    };
    return (React__default.createElement(Spin, { spinning: !getDurationTimeRecords ||
            loadingRecords ||
            loadingApproval ||
            loadingCreate ||
            loadingUpdate ||
            loadingRemove ||
            loadingCreateTimesheet },
        React__default.createElement(Row, { className: "toolBar" },
            React__default.createElement(Col, { xs: 24, md: 6, className: "control" },
                React__default.createElement(Button, { onClick: onClickToday }, " Today "),
                React__default.createElement(Button, { onClick: onClickBack }, " Back "),
                React__default.createElement(Button, { onClick: onClickNext }, " Next ")),
            React__default.createElement(Col, { xs: 24, md: 12, style: { textAlign: 'center' } },
                React__default.createElement("span", { className: "duration-start" },
                    " ",
                    moment(weekStart).format('MMMM DD')),
                " -",
                React__default.createElement("span", { className: "duration-end" }, moment(weekStart).format('MM') === moment(weekStart).add(1, 'week').format('MM')
                    ? moment(weekStart).add(1, 'week').format('DD')
                    : moment(weekStart).add(1, 'week').format('MMMM DD'))),
            React__default.createElement(Col, { xs: 24, md: 6, className: "control", style: { textAlign: 'right' } },
                React__default.createElement(Button, null, " Day "),
                React__default.createElement(Button, null, " Week "),
                React__default.createElement(Button, null, " Month "))),
        React__default.createElement(Row, null,
            React__default.createElement(Col, { xs: 24 },
                React__default.createElement(Card, null, selectedUser !== '__all' ? (React__default.createElement(TabularCalendar, { weekStart: weekStart, records: filterTimeRecords(timeRecords, {
                        selectedUser,
                        selectedProject,
                        members,
                    }).map((record) => (Object.assign(Object.assign({}, record), { projectId: record.projectId || '' }))), projects: projects, projectsMap: getProjectsMap(), projectsApproval: projectsApproval(), timesheets: memberTimesheets(selectedUser), selectedUser: selectedUser, members: members, loading: loadingRecords ||
                        loadingApproval ||
                        loadingCreate ||
                        loadingUpdate ||
                        loadingRemove ||
                        loadingCreateTimesheet, handleRemoveDuration: handleRemoveDuration, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, createTimesheet: createTimesheet })) : (members.map((mem) => {
                    return (React__default.createElement(TabularCalendar, { key: mem.userId, weekStart: weekStart, records: filterTimeRecords(timeRecords, {
                            selectedUser: mem.userId,
                            selectedProject,
                            members,
                        }).map((record) => (Object.assign(Object.assign({}, record), { projectId: record.projectId || '' }))), projects: projects, projectsMap: getProjectsMap(), projectsApproval: projectsApproval(), timesheets: memberTimesheets(mem.userId), selectedUser: mem.userId, members: members, loading: loadingRecords ||
                            loadingApproval ||
                            loadingCreate ||
                            loadingUpdate ||
                            loadingRemove ||
                            loadingCreateTimesheet, handleRemoveDuration: handleRemoveDuration, createTimeRecord: createTimeRecord, updateTimeRecord: updateTimeRecord, createTimesheet: createTimesheet }));
                })))))));
};export default TabularCalendarWrapper;//# sourceMappingURL=index.js.map
